
#########################################################################
#
#  $Id: Makefile.inc 1.1 2009/07/24 14:26:19 martin REL_M $
#
#  Description:
#    Common definitions included by the mbgtools-lx Makefiles.
#
# -----------------------------------------------------------------------
#  $Log: Makefile.inc $
#  Revision 1.1  2009/07/24 14:26:19  martin
#  Initial revision.
#
#########################################################################

ifeq ($(MBGLIB),)
  MBGLIB = ../mbglib
endif

ifeq ($(LOC_INCL),)
  LOC_INCL = -I. -I $(MBGLIB)/common -I $(MBGLIB)/linux
endif

CFLAGS += -Wall

ifneq ($(DEBUG),)
  CFLAGS += -g -DDEBUG=1
else
  CFLAGS += -O2
endif

CFLAGS += $(LOC_INCL)



# Macros useful to run individual commands as root:

ifeq ($(SUDO),)
  SUDO := $(shell which sudo)
endif

ifeq ($(SU),)
  SU := $(shell which su)
endif

UID = $(shell id -u)

run_as_root = $(shell echo "$(SUDO) $(1) || $(SU) -c \"$(1)\"" )
make_as_root = $(call run_as_root, make $(1))



# the next stuff is only useful if not building kernel module:
ifeq ($(K26),)

ifndef MBGDEVIO_SIMPLE
  MBGDEVIO_SIMPLE = 1
endif

ifeq ($(MBGDEVIO_SIMPLE),1)
  CFLAGS += -DMBGDEVIO_SIMPLE=1
endif


ifdef USE_THREAD_API

# Check whether thread affinity is supported by the installed pthread library.
# Newer versions of glibc/pthread (at least glibc 2.5) supports this natively.
# Older versions of glibc/pthread (e.g. glibc 2.3) may not support this natively
# but may provide a NPTL (New Posix Thread Library) library located under separate 
# include and lib paths. We try to detect NPTL and add those paths to the search 
# paths only if USE_NTPL has been defined e.g. on the make command line.

# Check whether pthread_getaffinity is supported by the standard pthread.h
USE_THREAD_AFFINITY := $(shell grep -q pthread_getaffinity /usr/include/pthread.h && echo 1)

# If it is not, check nptl/pthread.h
ifeq ($(USE_THREAD_AFFINITY),)
  ifneq ($(USE_NPTL),)
    USE_THREAD_AFFINITY := $(shell grep -q pthread_getaffinity /usr/include/nptl/pthread.h && echo 1)
  endif

  ifeq ($(USE_THREAD_AFFINITY),)
    # If not found either, don't support thread affinity
    USE_THREAD_AFFINITY=0
  else
    #if supported via nptl add the associated search paths
    CFLAGS += -I /usr/include/nptl -L /usr/lib/nptl
  endif
endif

# May want to use the result for our program
## CFLAGS += -DUSE_THREAD_AFFINITY=$(USE_THREAD_AFFINITY)

CFLAGS += -DMBGDEVIO_USE_THREAD_API=1

LDFLAGS += -lpthread

endif


# common targets:

all: $(TARGET)

$(TARGET): $(SRCS) $(INCL)
	$(CC) $(CFLAGS) $(LDFLAGS) -o $@ $(SRCS)

install:
ifneq ($(UID),0)
	$(call make_as_root, $@)
else
	cp $(TARGET) $(INST_DIR)
endif

uninstall:
ifneq ($(UID),0)
	$(call make_as_root, $@)
else
	rm -f $(INST_DIR)/$(TARGET)
endif

clean:
	rm -f *.o *~ core
	rm -f $(TARGET)

distclean: clean

endif
