
/**************************************************************************
 *
 *  $Id: mbggenio.h 1.5 2008/12/05 13:27:33 martin REL_M $
 *
 *  Copyright (c) Meinberg Funkuhren, Bad Pyrmont, Germany
 *
 *  Description:
 *    Definitions for generic port I/O.
 *
 * -----------------------------------------------------------------------
 *  $Log: mbggenio.h $
 *  Revision 1.5  2008/12/05 13:27:33  martin
 *  Generally put macro arguments in brackets for evaluation
 *  to avoid potential side effects.
 *  There has been a problem with an improper written outp() macro
 *  in the Borland C 3.1 library's conio.h file.
 *  Support mapped I/O resources.
 *  Revision 1.4  2008/02/05 13:38:57  martin
 *  Added support for QNX.
 *  Revision 1.3  2007/03/21 14:48:56  martin
 *  Use standard inp(), outp() also under Windows since the generic
 *  Windows functions READ_PORT_UCHA(), etc., are not very
 *  compatible across DDK versions.
 *  Revision 1.2  2007/03/02 10:23:34Z  martin
 *  Renamed generic port I/O macros.
 *  Fully support Linux, *BSD, Windows, NetWare, DOS, and OS/2.
 *  Revision 1.1  2006/09/20 10:47:21  martin
 *
 **************************************************************************/

#ifndef _MBGGENIO_H
#define _MBGGENIO_H


/* Other headers to be included */

#include <mbg_arch.h>


/* Start of header body */

#ifdef __cplusplus
extern "C" {
#endif


#if defined( MBG_TGT_LINUX )

  #if MBG_USE_MM_IO_FOR_PCI
    #define _mbg_inp8( _d, _p )        ( readb( (_p) ) )
    #define _mbg_inp16( _d, _p )       ( readw( (_p) ) )
    #define _mbg_inp32( _d, _p )       ( readl( (_p) ) )

    #define _mbg_outp8( _d, _p, _v )   writeb( (_v), (_p) )
    #define _mbg_outp16( _d, _p, _v )  writew( (_v), (_p) )
    #define _mbg_outp32( _d, _p, _v )  writel( (_v), (_p) )
  #else
    #define _mbg_inp8( _d, _p )        ( (uint8_t) inb( (_p) ) )
    #define _mbg_inp16( _d, _p )       ( (uint16_t) inw( (_p) ) )
    #define _mbg_inp32( _d, _p )       ( (uint32_t) inl( (_p) ) )

    #define _mbg_outp8( _d, _p, _v )   outb( (_v), (_p) )
    #define _mbg_outp16( _d, _p, _v )  outw( (_v), (_p) )
    #define _mbg_outp32( _d, _p, _v )  outl( (_v), (_p) )
  #endif

#elif defined( MBG_TGT_BSD )

  #include <sys/types.h>
  #include <sys/bus.h>
  #include <machine/bus.h>

  #define _mbg_inp8( _d, _p )        ( (uint8_t) bus_space_read_1( ((_d)->bst), ((_d)->bsh), (_p) ) )
  #define _mbg_inp16( _d, _p )       ( (uint16_t) bus_space_read_2( ((_d)->bst), ((_d)->bsh), (_p) ) )
  #define _mbg_inp32( _d, _p )       ( (uint32_t) bus_space_read_4( ((_d)->bst), ((_d)->bsh), (_p) ) )

  #define _mbg_outp8( _d, _p, _v )   bus_space_write_1( ((_d)->bst), ((_d)->bsh), (_p), (_v) )
  #define _mbg_outp16( _d, _p, _v )  bus_space_write_2( ((_d)->bst), ((_d)->bsh), (_p), (_v) )
  #define _mbg_outp32( _d, _p, _v )  bus_space_write_4( ((_d)->bst), ((_d)->bsh), (_p), (_v) )

#elif defined( MBG_TGT_WIN32 )

  #define _mbg_inp8( _d, _p )        ( (uint8_t) inp( (_p) ) )
  #define _mbg_inp16( _d, _p )       ( (uint16_t) inpw( (_p) ) )
  #define _mbg_inp32( _d, _p )       ( (uint32_t) inpd( (_p) ) )

  #define _mbg_outp8( _d, _p, _v )   outp( (_p), (_v) )
  #define _mbg_outp16( _d, _p, _v )  outpw( (_p), (_v) )
  #define _mbg_outp32( _d, _p, _v )  outpd( (_p), (_v) )

#elif defined( MBG_TGT_DOS ) || defined( MBG_TGT_NETWARE ) || defined( MBG_TGT_OS2 )

  #include <conio.h>

  #if defined( MBG_TGT_DOS ) || defined( MBG_TGT_OS2 )
    #include <xportio.h>
  #endif

  #define _mbg_inp8( _d, _p )        ( (uint8_t) inp( (_p) ) )
  #define _mbg_inp16( _d, _p )       ( (uint16_t) inpw( (_p) ) )
  #define _mbg_inp32( _d, _p )       ( (uint32_t) inpd( (_p) ) )

  #define _mbg_outp8( _d, _p, _v )   outp( (_p), (_v) )
  #define _mbg_outp16( _d, _p, _v )  outpw( (_p), (_v) )
  #define _mbg_outp32( _d, _p, _v )  outpd( (_p), (_v) )

#elif defined( MBG_TGT_QNX )

  #if defined( MBG_TGT_QNX_NTO )   // compiling for QNX Neutrino
    // don't know if we have to distinguish between different compilers
    #include <hw/inout.h>

    // ATTENTION: mmap_device_io() must be called on non-x86 architectures
    // to remap the ports, otherwise a segmentation fault will occur if
    // the port I/O functions are being called.

    #define _mbg_inp8( _d, _p )        ( (uint8_t) in8( (_p) ) )
    #define _mbg_inp16( _d, _p )       ( (uint16_t) in16( (_p) ) )
    #define _mbg_inp32( _d, _p )       ( (uint32_t) in32( (_p) ) )

    #define _mbg_outp8( _d, _p, _v )   out8( (_p), (_v) )
    #define _mbg_outp16( _d, _p, _v )  out16( (_p), (_v) )
    #define _mbg_outp32( _d, _p, _v )  out32( (_p), (_v) )

  #else // compiling for QNX 4

    #if defined( __WATCOMC__ )   // using Watcom C

      // Include prototypes of port I/O functions
      // which should match the calls used in the mbglib functions.
      #include <conio.h>

      #define _mbg_inp8( _d, _p )        ( (uint8_t) inp( (_p) ) )
      #define _mbg_inp16( _d, _p )       ( (uint16_t) inpw( (_p) ) )
      #define _mbg_inp32( _d, _p )       ( (uint32_t) inpd( (_p) ) )

      #define _mbg_outp8( _d, _p, _v )   outp( (_p), (_v) )
      #define _mbg_outp16( _d, _p, _v )  outpw( (_p), (_v) )
      #define _mbg_outp32( _d, _p, _v )  outpd( (_p), (_v) )

    #endif

  #endif

#endif



#define _mbg_inp16_to_cpu( _d, _p )       _mbg16_to_cpu( _mbg_inp16( (_d), (_p) ) )
#define _mbg_inp32_to_cpu( _d, _p )       _mbg32_to_cpu( _mbg_inp32( (_d), (_p) ) )

#define _mbg_outp16_to_mbg( _d, _p, _v )  _mbg_outp16( (_d), (_p), _cpu_to_mbg16( (_v) ) )
#define _mbg_outp32_to_mbg( _d, _p, _v )  _mbg_outp32( (_d), (_p), _cpu_to_mbg32( (_v) ) )



#ifdef __cplusplus
}
#endif

/* End of header body */

#endif  /* _MBGGENIO_H */
