
/**************************************************************************
 *
 *  $Id: mbgioctl.h 1.23 2009/09/29 15:08:41 martin REL_M $
 *
 *  Copyright (c) Meinberg Funkuhren, Bad Pyrmont, Germany
 *
 *  Description:
 *    Definitions used with device driver IOCTL.
 *
 * -----------------------------------------------------------------------
 *  $Log: mbgioctl.h $
 *  Revision 1.23  2009/09/29 15:08:41  martin
 *  Support retrieving time discipline info.
 *  Revision 1.22  2009/08/17 13:48:17  martin
 *  Moved specific definition of symbol _HAVE_IOCTL_WITH_SIZE from
 *  mbgdevio.c here and renamed it to _MBG_SUPP_VAR_ACC_SIZE.
 *  Revision 1.21  2009/06/19 12:18:53  martin
 *  Added PCPS_GIVE_IRIG_TIME command and associated definitions.
 *  Fixed a declaration which might have led to syntax errors.
 *  Revision 1.20  2009/06/09 10:02:36Z  daniel
 *  Support PTP configuration and state.
 *  Support simple LAN interface configuration.
 *  Revision 1.19  2009/03/19 15:17:59  martin
 *  Support reading MM timestamps without cycles.
 *  Support UTC parms and configurable time scales.
 *  For consistent naming renamed IOCTL_GET_FAST_HR_TIMESTAMP
 *  to IOCTL_GET_FAST_HR_TIMESTAMP_CYCLES.
 *  Added IOCTL_DEV_HAS_IRIG_CTRL_BITS and IOCTL_GET_IRIG_CTRL_BITS.
 *  Revision 1.18  2008/12/11 10:32:56Z  martin
 *  Added _cmd_from_ioctl_code() macro for Linux.
 *  Added IOCTL codes for .._has_asic_version() and .._has_asic_features().
 *  Added IOCTL codes for ..._is_msf(), .._is_lwr(), .._is_wwvb().
 *  Added IOCTL codes IOCTL_GET_IRQ_STAT_INFO, IOCTL_GET_CYCLES_FREQUENCY,
 *  IOCTL_HAS_FAST_HR_TIMESTAMP, and IOCTL_GET_FAST_HR_TIMESTAMP.
 *  Revision 1.17  2008/01/17 09:35:15  daniel
 *  Added ioctl calls IOCTL_GET_MAPPED_MEM_ADDR and
 *  IOCTL_UNMAP_MAPPED_MEM.
 *  Cleanup for PCI ASIC version and features.
 *  Revision 1.16  2007/09/25 10:37:04Z  martin
 *  Added macro _cmd_from_ioctl_code() for Windows.
 *  Revision 1.15  2007/05/21 15:00:01Z  martin
 *  Unified naming convention for symbols related to ref_offs.
 *  Revision 1.14  2007/03/02 10:27:03  martin
 *  Preliminary support for *BSD.
 *  Preliminary _cmd_from_ioctl().
 *  Revision 1.13  2006/03/10 10:36:54  martin
 *  Added support for programmable pulse outputs.
 *  Revision 1.12  2005/06/02 10:22:05Z  martin
 *  Added IOCTL code IOCTL_GET_SYNTH_STATE.
 *  Added IOCTL codes IOCTL_DEV_HAS_GENERIC_IO, 
 *  IOCTL_PCPS_GENERIC_IO, and IOCTL_GET_SYNTH_STATE.
 *  Revision 1.11  2005/01/14 10:21:11Z  martin
 *  Added IOCTLs which query device features.
 *  Revision 1.10  2004/12/09 11:03:36Z  martin
 *  Support configuration of on-board frequency synthesizer.
 *  Revision 1.9  2004/11/09 12:49:41Z  martin
 *  Modifications were required in order to be able to configure IRIG 
 *  settings of cards which provide both IRIG input and output.
 *  The existing codes have been renamed with .._RX.. and are used to 
 *  configure the IRIG receiver (input). New codes have been defined 
 *  used to configure the IRIG transmitter.
 *  Renamed IOCTL_GET_GPS_STAT to IOCTL_GET_GPS_BVAR_STAT.
 *  Use more specific data types than generic types.
 *  Modified IOCTL codes used for hardware debugging.
 *  Revision 1.8  2004/09/06 15:46:04Z  martin
 *  Changed definition of IOCTL codes to support syntax used 
 *  with Linux kernel 2.6.x.
 *  Account for renamed symbols.
 *  Revision 1.7  2004/04/07 10:08:11  martin
 *  Added IOCTL codes used to trigger hardware debug events.
 *  Revision 1.6  2003/12/22 15:37:18Z  martin
 *  Added codes to read ASIC version, and read times
 *  with associated cycle counter values.
 *  Revision 1.5  2003/06/19 09:02:30Z  martin
 *  New codes IOCTL_GET_PCPS_UCAP_ENTRIES and IOCTL_GET_PCPS_UCAP_EVENT.
 *  Renamed IOCTL_PCPS_CLR_CAP_BUFF to IOCTL_PCPS_CLR_UCAP_BUFF.
 *  Cleaned up IOCTL code names related to PCPS_TZDL.
 *  Reordered upper IOCTL code numbers again.
 *  Revision 1.4  2003/04/09 13:50:39Z  martin
 *  Re-organized IOCTL codes.
 *  Supports Win32.
 *  Added missing pragma pack().
 *  Revision 1.3  2003/02/14 13:20:08Z  martin
 *  Include mbggeo.h instead of mygeo.h.
 *  Revision 1.2  2001/11/30 09:52:48  martin
 *  Added support for event_time which, however, requires
 *  a custom GPS firmware.
 *  Revision 1.1  2001/03/05 16:34:22  MARTIN
 *  Initial revision
 *
 **************************************************************************/

#ifndef _MBGIOCTL_H
#define _MBGIOCTL_H


/* Other headers to be included */

#include <mbg_tgt.h>
#include <pcpsdev.h>
#include <mbggeo.h>
#include <use_pack.h>

#if defined( MBG_TGT_LINUX )

  #include <linux/ioctl.h>

  // a magic number used to generate IOCTL cmd codes
  #define IOTYPE 'M'

  #define _MBG_IO   _IO
  #define _MBG_IOR  _IOR
  #define _MBG_IOW  _IOW

  #define _cmd_from_ioctl_code( _ioc ) \
    _IOC_NR( _ioc )

#elif defined( MBG_TGT_BSD )

  #include <sys/ioccom.h>

  // a magic number used to generate IOCTL cmd codes
  #define IOTYPE 'M'

  #define _MBG_IO   _IO
  #define _MBG_IOR  _IOR
  #define _MBG_IOW  _IOW

#elif defined( MBG_TGT_WIN32 )

  #if !defined( _MBG_SUPP_VAR_ACC_SIZE )
    // Windows supports IOCTL commands where the sizes of
    // input and output buffer can be specified dynamically.
    #define _MBG_SUPP_VAR_ACC_SIZE   1
  #endif

  #ifndef _KDD_   // must be defined in the KDDs' SOURCES file
    #include <windows.h>
    #include <winioctl.h>
  #endif

  #if !defined( MBG_TGT_WIN32_NON_PNP )
    #ifdef _MBGIOCTL
      #include <initguid.h>   // instance the GUID
    #else
      #include <guiddef.h>    // just define the GUID
    #endif
  #endif

  #ifdef DEFINE_GUID   // don't break compiles of drivers that
                       // include this header but don't want the
                       // GUIDs

    // ClassGuid = { 78A1C341-4539-11d3-B88D-00C04FAD5171 }
    DEFINE_GUID( GUID_MEINBERG_DEVICE,
                 0x78A1C341L, 0x4539, 0x11D3,
                 0xB8, 0x8D, 0x00, 0xC0, 0x4F, 0xAD, 0x51, 0x71 );
  #endif

  // Device type in the "User Defined" range."
  #define PCPS_TYPE 40000

  // IOCTL function codes from 0x800 to 0xFFF are for customer use.
  #define _MBG_IOCTL_BIAS  0x930

  #define _MBG_IO( _t, _n ) \
    CTL_CODE( PCPS_TYPE, _MBG_IOCTL_BIAS + _n, METHOD_BUFFERED, FILE_READ_ACCESS )

  #define _MBG_IOR( _t, _n, _sz ) \
    _MBG_IO( _t, _n )

  #define _MBG_IOW  _MBG_IOR

  #define _cmd_from_ioctl_code( _ioc ) \
    ( ( ( (_ioc) >> 2 ) & 0x0FFF ) - _MBG_IOCTL_BIAS )

#endif


#if !defined( _MBG_SUPP_VAR_ACC_SIZE )
  // Many operating systems don't support specifying the sizes of IOCTL
  // input and output buffers dynamically, so we disable this by default.
  #define _MBG_SUPP_VAR_ACC_SIZE   0
#endif


#ifdef _MBGIOCTL
 #define _ext
 #define _DO_INIT
#else
 #define _ext extern
#endif


#if defined( _USE_PACK )   // set byte alignment
  #pragma pack( 1 )
#endif


/* Start of header body */

// read general driver info, device info, and status port
#define IOCTL_GET_PCPS_DRVR_INFO         _MBG_IOR( IOTYPE, 0x00, PCPS_DRVR_INFO )
#define IOCTL_GET_PCPS_DEV               _MBG_IOR( IOTYPE, 0x01, PCPS_DEV )
#define IOCTL_GET_PCPS_STATUS_PORT       _MBG_IOR( IOTYPE, 0x02, PCPS_STATUS_PORT )

// generic read/write operations (preliminary)
#define IOCTL_PCPS_GENERIC_READ          _MBG_IO(  IOTYPE, 0x03 )
#define IOCTL_PCPS_GENERIC_WRITE         _MBG_IO(  IOTYPE, 0x04 )
#define IOCTL_PCPS_GENERIC_READ_GPS      _MBG_IO(  IOTYPE, 0x05 )
#define IOCTL_PCPS_GENERIC_WRITE_GPS     _MBG_IO(  IOTYPE, 0x06 )

// normal direct read/write operations
#define IOCTL_GET_PCPS_TIME              _MBG_IOR( IOTYPE, 0x10, PCPS_TIME )
#define IOCTL_SET_PCPS_TIME              _MBG_IOW( IOTYPE, 0x11, PCPS_STIME )

#define IOCTL_GET_PCPS_SYNC_TIME         _MBG_IOR( IOTYPE, 0x12, PCPS_TIME )

#define IOCTL_GET_PCPS_TIME_SEC_CHANGE   _MBG_IOR( IOTYPE, 0x13, PCPS_TIME )

#define IOCTL_GET_PCPS_HR_TIME           _MBG_IOR( IOTYPE, 0x14, PCPS_HR_TIME )

// the next one is supported with custom GPS firmware only:
#define IOCTL_SET_PCPS_EVENT_TIME        _MBG_IOW( IOTYPE, 0x15, PCPS_TIME_STAMP )

#define IOCTL_GET_PCPS_SERIAL            _MBG_IOR( IOTYPE, 0x16, PCPS_SERIAL )
#define IOCTL_SET_PCPS_SERIAL            _MBG_IOW( IOTYPE, 0x17, PCPS_SERIAL )

#define IOCTL_GET_PCPS_TZCODE            _MBG_IOR( IOTYPE, 0x18, PCPS_TZCODE )
#define IOCTL_SET_PCPS_TZCODE            _MBG_IOW( IOTYPE, 0x19, PCPS_TZCODE )

#define IOCTL_GET_PCPS_TZDL              _MBG_IOR( IOTYPE, 0x1A, PCPS_TZDL )
#define IOCTL_SET_PCPS_TZDL              _MBG_IOW( IOTYPE, 0x1B, PCPS_TZDL )

#define IOCTL_GET_REF_OFFS               _MBG_IOR( IOTYPE, 0x1C, MBG_REF_OFFS )
#define IOCTL_SET_REF_OFFS               _MBG_IOW( IOTYPE, 0x1D, MBG_REF_OFFS )

#define IOCTL_GET_MBG_OPT_INFO           _MBG_IOR( IOTYPE, 0x1E, MBG_OPT_INFO )
#define IOCTL_SET_MBG_OPT_SETTINGS       _MBG_IOW( IOTYPE, 0x1F, MBG_OPT_SETTINGS )

#define IOCTL_GET_PCPS_IRIG_RX_INFO      _MBG_IOR( IOTYPE, 0x20, IRIG_INFO )
#define IOCTL_SET_PCPS_IRIG_RX_SETTINGS  _MBG_IOW( IOTYPE, 0x21, IRIG_SETTINGS )

#define IOCTL_PCPS_CLR_UCAP_BUFF         _MBG_IO(  IOTYPE, 0x22 )
#define IOCTL_GET_PCPS_UCAP_ENTRIES      _MBG_IOR( IOTYPE, 0x23, PCPS_UCAP_ENTRIES )
#define IOCTL_GET_PCPS_UCAP_EVENT        _MBG_IOR( IOTYPE, 0x24, PCPS_HR_TIME )


#define IOCTL_GET_GPS_TZDL               _MBG_IOR( IOTYPE, 0x25, TZDL )
#define IOCTL_SET_GPS_TZDL               _MBG_IOW( IOTYPE, 0x26, TZDL )

#define IOCTL_GET_GPS_SW_REV             _MBG_IOR( IOTYPE, 0x27, SW_REV )

#define IOCTL_GET_GPS_BVAR_STAT          _MBG_IOR( IOTYPE, 0x28, BVAR_STAT )

#define IOCTL_GET_GPS_TIME               _MBG_IOR( IOTYPE, 0x29, TTM )
#define IOCTL_SET_GPS_TIME               _MBG_IOW( IOTYPE, 0x2A, TTM )

#define IOCTL_GET_GPS_PORT_PARM          _MBG_IOR( IOTYPE, 0x2B, PORT_PARM )
#define IOCTL_SET_GPS_PORT_PARM          _MBG_IOW( IOTYPE, 0x2C, PORT_PARM )

#define IOCTL_GET_GPS_ANT_INFO           _MBG_IOR( IOTYPE, 0x2D, ANT_INFO )

#define IOCTL_GET_GPS_UCAP               _MBG_IOR( IOTYPE, 0x2E, TTM )

#define IOCTL_GET_GPS_ENABLE_FLAGS       _MBG_IOR( IOTYPE, 0x2F, ENABLE_FLAGS )
#define IOCTL_SET_GPS_ENABLE_FLAGS       _MBG_IOW( IOTYPE, 0x30, ENABLE_FLAGS )

#define IOCTL_GET_GPS_STAT_INFO          _MBG_IOR( IOTYPE, 0x31, STAT_INFO )

#define IOCTL_SET_GPS_CMD                _MBG_IOW( IOTYPE, 0x32, GPS_CMD )

#define IOCTL_GET_GPS_IDENT              _MBG_IOR( IOTYPE, 0x33, IDENT )

#define IOCTL_GET_GPS_POS                _MBG_IOR( IOTYPE, 0x34, POS )
#define IOCTL_SET_GPS_POS_XYZ            _MBG_IOW( IOTYPE, 0x35, XYZ )
#define IOCTL_SET_GPS_POS_LLA            _MBG_IOW( IOTYPE, 0x36, LLA )

#define IOCTL_GET_GPS_ANT_CABLE_LEN      _MBG_IOR( IOTYPE, 0x37, ANT_CABLE_LEN )
#define IOCTL_SET_GPS_ANT_CABLE_LEN      _MBG_IOW( IOTYPE, 0x38, ANT_CABLE_LEN )

#define IOCTL_GET_GPS_RECEIVER_INFO      _MBG_IOR( IOTYPE, 0x39, RECEIVER_INFO )
#define IOCTL_GET_GPS_ALL_STR_TYPE_INFO  _MBG_IO(  IOTYPE, 0x3A )  // size variable
#define IOCTL_GET_GPS_ALL_PORT_INFO      _MBG_IO(  IOTYPE, 0x3B )  // size variable

#define IOCTL_SET_GPS_PORT_SETTINGS_IDX  _MBG_IOW( IOTYPE, 0x3C, PORT_SETTINGS_IDX )

#define IOCTL_GET_PCI_ASIC_VERSION       _MBG_IOR( IOTYPE, 0x3D, PCI_ASIC_VERSION )

#define IOCTL_GET_PCPS_TIME_CYCLES       _MBG_IOR( IOTYPE, 0x3E, PCPS_TIME_CYCLES )
#define IOCTL_GET_PCPS_HR_TIME_CYCLES    _MBG_IOR( IOTYPE, 0x3F, PCPS_HR_TIME_CYCLES )

#define IOCTL_GET_PCPS_IRIG_TX_INFO      _MBG_IOR( IOTYPE, 0x40, IRIG_INFO )
#define IOCTL_SET_PCPS_IRIG_TX_SETTINGS  _MBG_IOW( IOTYPE, 0x41, IRIG_SETTINGS )

#define IOCTL_GET_SYNTH                  _MBG_IOR( IOTYPE, 0x42, SYNTH )
#define IOCTL_SET_SYNTH                  _MBG_IOW( IOTYPE, 0x43, SYNTH )


#define IOCTL_DEV_IS_GPS                 _MBG_IOR( IOTYPE, 0x44, int )
#define IOCTL_DEV_IS_DCF                 _MBG_IOR( IOTYPE, 0x45, int )
#define IOCTL_DEV_IS_IRIG_RX             _MBG_IOR( IOTYPE, 0x46, int )

#define IOCTL_DEV_HAS_HR_TIME            _MBG_IOR( IOTYPE, 0x47, int )
#define IOCTL_DEV_HAS_CAB_LEN            _MBG_IOR( IOTYPE, 0x48, int )
#define IOCTL_DEV_HAS_TZDL               _MBG_IOR( IOTYPE, 0x49, int )
#define IOCTL_DEV_HAS_PCPS_TZDL          _MBG_IOR( IOTYPE, 0x4A, int )
#define IOCTL_DEV_HAS_TZCODE             _MBG_IOR( IOTYPE, 0x4B, int )
#define IOCTL_DEV_HAS_TZ                 _MBG_IOR( IOTYPE, 0x4C, int )
#define IOCTL_DEV_HAS_EVENT_TIME         _MBG_IOR( IOTYPE, 0x4D, int )
#define IOCTL_DEV_HAS_RECEIVER_INFO      _MBG_IOR( IOTYPE, 0x4E, int )
#define IOCTL_DEV_CAN_CLR_UCAP_BUFF      _MBG_IOR( IOTYPE, 0x4F, int )
#define IOCTL_DEV_HAS_UCAP               _MBG_IOR( IOTYPE, 0x50, int )
#define IOCTL_DEV_HAS_IRIG_TX            _MBG_IOR( IOTYPE, 0x51, int )
#define IOCTL_DEV_HAS_SERIAL_HS          _MBG_IOR( IOTYPE, 0x52, int )
#define IOCTL_DEV_HAS_SIGNAL             _MBG_IOR( IOTYPE, 0x53, int )
#define IOCTL_DEV_HAS_MOD                _MBG_IOR( IOTYPE, 0x54, int )
#define IOCTL_DEV_HAS_IRIG               _MBG_IOR( IOTYPE, 0x55, int )
#define IOCTL_DEV_HAS_REF_OFFS           _MBG_IOR( IOTYPE, 0x56, int )
#define IOCTL_DEV_HAS_OPT_FLAGS          _MBG_IOR( IOTYPE, 0x57, int )
#define IOCTL_DEV_HAS_GPS_DATA           _MBG_IOR( IOTYPE, 0x58, int )
#define IOCTL_DEV_HAS_SYNTH              _MBG_IOR( IOTYPE, 0x59, int )
#define IOCTL_DEV_HAS_GENERIC_IO         _MBG_IOR( IOTYPE, 0x5A, int )

#define IOCTL_PCPS_GENERIC_IO            _MBG_IO(  IOTYPE, 0x5B )

#define IOCTL_GET_SYNTH_STATE            _MBG_IOR( IOTYPE, 0x5C, SYNTH_STATE )

#define IOCTL_GET_GPS_ALL_POUT_INFO      _MBG_IO(  IOTYPE, 0x5D )  // size variable
#define IOCTL_SET_GPS_POUT_SETTINGS_IDX  _MBG_IOW( IOTYPE, 0x5E, POUT_SETTINGS_IDX )

#define IOCTL_GET_MAPPED_MEM_ADDR        _MBG_IOR( IOTYPE, 0x5F, PCPS_MAPPED_MEM )
#define IOCTL_UNMAP_MAPPED_MEM           _MBG_IOR( IOTYPE, 0x60, PCPS_MAPPED_MEM )

#define IOCTL_GET_PCI_ASIC_FEATURES      _MBG_IOR( IOTYPE, 0x61, PCI_ASIC_FEATURES )

#define IOCTL_HAS_PCI_ASIC_FEATURES      _MBG_IOR( IOTYPE, 0x62, int )
#define IOCTL_HAS_PCI_ASIC_VERSION       _MBG_IOR( IOTYPE, 0x63, int )

#define IOCTL_DEV_IS_MSF                 _MBG_IOR( IOTYPE, 0x64, int )
#define IOCTL_DEV_IS_LWR                 _MBG_IOR( IOTYPE, 0x65, int )
#define IOCTL_DEV_IS_WWVB                _MBG_IOR( IOTYPE, 0x66, int )

#define IOCTL_GET_IRQ_STAT_INFO          _MBG_IOR( IOTYPE, 0x67, PCPS_IRQ_STAT_INFO )
#define IOCTL_GET_CYCLES_FREQUENCY       _MBG_IOR( IOTYPE, 0x68, MBG_PC_CYCLES_FREQUENCY )

#define IOCTL_HAS_FAST_HR_TIMESTAMP         _MBG_IOR( IOTYPE, 0x69, int )
#define IOCTL_GET_FAST_HR_TIMESTAMP_CYCLES  _MBG_IOR( IOTYPE, 0x6A, PCPS_TIME_STAMP_CYCLES )
#define IOCTL_GET_FAST_HR_TIMESTAMP         _MBG_IOR( IOTYPE, 0x6B, PCPS_TIME_STAMP )

#define IOCTL_DEV_HAS_GPS_TIME_SCALE        _MBG_IOR( IOTYPE, 0x6C, int )
#define IOCTL_GET_GPS_TIME_SCALE_INFO       _MBG_IOR( IOTYPE, 0x6D, MBG_TIME_SCALE_INFO )
#define IOCTL_SET_GPS_TIME_SCALE_SETTINGS   _MBG_IOW( IOTYPE, 0x6E, MBG_TIME_SCALE_SETTINGS )

#define IOCTL_DEV_HAS_GPS_UTC_PARM       _MBG_IOR( IOTYPE, 0x6F, int )
#define IOCTL_GET_GPS_UTC_PARM           _MBG_IOR( IOTYPE, 0x70, UTC )
#define IOCTL_SET_GPS_UTC_PARM           _MBG_IOW( IOTYPE, 0x71, UTC )

#define IOCTL_DEV_HAS_IRIG_CTRL_BITS     _MBG_IOR( IOTYPE, 0x72, int )
#define IOCTL_GET_IRIG_CTRL_BITS         _MBG_IOR( IOTYPE, 0x73, MBG_IRIG_CTRL_BITS )

#define IOCTL_DEV_HAS_LAN_INTF           _MBG_IOR( IOTYPE, 0x74, int )
#define IOCTL_GET_LAN_IF_INFO            _MBG_IOR( IOTYPE, 0x75, LAN_IF_INFO )
#define IOCTL_GET_IP4_STATE              _MBG_IOR( IOTYPE, 0x76, IP4_SETTINGS )
#define IOCTL_GET_IP4_SETTINGS           _MBG_IOR( IOTYPE, 0x77, IP4_SETTINGS )
#define IOCTL_SET_IP4_SETTINGS           _MBG_IOW( IOTYPE, 0x78, IP4_SETTINGS )

#define IOCTL_DEV_IS_PTP                 _MBG_IOR( IOTYPE, 0x79, int )
#define IOCTL_DEV_HAS_PTP                _MBG_IOR( IOTYPE, 0x7A, int )
#define IOCTL_GET_PTP_STATE              _MBG_IOR( IOTYPE, 0x7B, PTP_STATE )
#define IOCTL_GET_PTP_CFG_INFO           _MBG_IOR( IOTYPE, 0x7C, PTP_CFG_INFO )
#define IOCTL_SET_PTP_CFG_SETTINGS       _MBG_IOW( IOTYPE, 0x7D, PTP_CFG_SETTINGS )

#define IOCTL_DEV_HAS_IRIG_TIME          _MBG_IOR( IOTYPE, 0x7E, int )
#define IOCTL_GET_IRIG_TIME              _MBG_IOR( IOTYPE, 0x7F, PCPS_IRIG_TIME )

#define IOCTL_GET_TIME_INFO_HRT          _MBG_IOR( IOTYPE, 0x80, MBG_TIME_INFO_HRT )
#define IOCTL_GET_TIME_INFO_TSTAMP       _MBG_IOR( IOTYPE, 0x81, MBG_TIME_INFO_TSTAMP )

// The codes below are subject to changes without notice. They may be supported
// by some kernel drivers, but usage is restricted to Meinberg software development.
// Unrestricted usage may cause system malfunction !!
#define IOCTL_MBG_DBG_GET_PORT_ADDR      _MBG_IOR( IOTYPE, 0xF0, uint16_t )
#define IOCTL_MBG_DBG_SET_PORT_ADDR      _MBG_IOW( IOTYPE, 0xF1, uint16_t )
#define IOCTL_MBG_DBG_SET_BIT            _MBG_IOW( IOTYPE, 0xF2, uint8_t )
#define IOCTL_MBG_DBG_CLR_BIT            _MBG_IOW( IOTYPE, 0xF3, uint8_t )
#define IOCTL_MBG_DBG_CLR_ALL            _MBG_IO(  IOTYPE, 0xF4 )


#if !defined( _cmd_from_ioctl_code )
  #define _cmd_from_ioctl_code( _ioctl_code )   _ioctl_code
#endif



// The structure below is used by the IOCTL_PCPS_GENERIC_... calls.
typedef struct
{
  uint32_t info;
  uint32_t data_size_in;
  uint32_t data_size_out;
} IOCTL_GENERIC_CTL;


typedef struct
{
  IOCTL_GENERIC_CTL ctl;
  uint8_t data[1];
} IOCTL_GENERIC_BUFFER;



/* End of header body */

#undef _ext
#undef _DO_INIT


/* function prototypes: */

#ifdef __cplusplus
extern "C" {
#endif

/* ----- function prototypes begin ----- */

/* This section was generated automatically */
/* by MAKEHDR, do not remove the comments. */

/* (no header definitions found) */

/* ----- function prototypes end ----- */

#ifdef __cplusplus
}
#endif


#if defined( _USE_PACK )   // set byte alignment
  #pragma pack()
#endif

#endif  /* _MBGIOCTL_H */
