
/**************************************************************************
 *
 *  $Id: mbg_lx.h 1.8 2009/03/13 09:04:18 martin REL_M $
 *
 *  Copyright (c) Meinberg Funkuhren, Bad Pyrmont, Germany
 *
 *  Description:
 *    OS dependend definitions/redefinitions for Linux.
 *
 * -----------------------------------------------------------------------
 *  $Log: mbg_lx.h $
 *  Revision 1.8  2009/03/13 09:04:18  martin
 *  Added default definition for _PCPS_USB_FULL_CYCLIC_INTV.
 *  Revision 1.7  2008/12/19 15:22:05  martin
 *  Moved lots of code here which provides compatibility across
 *  kernel versions.
 *  Added preliminary check if kthread API can be used. However,
 *  the exact version number still needs to be verified.
 *  Added a check whether USB timeouts have to be specified
 *  in jiffies (old) or milliseconds (newer).
 *  Revision 1.6  2007/02/22 15:26:56  martin
 *  Include linux/autoconf.h rather than linux/config.h since the latter has been
 *  a stub for autoconf.h which has been removed around kernel version 2.6.19.
 *  Revision 1.5  2006/09/20 12:09:29  martin
 *  Removed port I/O function redefinitions which have been replaced
 *  by generic function definitions in mbggenio.h.
 *  Revision 1.4  2006/07/07 12:06:40  martin
 *  Include modversions.h if required by the kernel.
 *  Revision 1.3  2001/05/31 09:21:09  martin
 *  #include <asm/io.h> instead of <sys/io.h>
 *  Revision 1.2  2001/03/05 16:36:11  MARTIN
 *  Renamed file.
 *
 **************************************************************************/

#ifndef _MBG_LX_H
#define _MBG_LX_H


#include <linux/version.h>

#if !defined( KERNEL_VERSION ) || ( LINUX_VERSION_CODE < KERNEL_VERSION( 2, 2, 0 ) )
  #error "You need to use at least a v2.2.x Linux kernel."
#endif

#if ( LINUX_VERSION_CODE < KERNEL_VERSION( 2, 6, 0 ) )
  #define __NO_VERSION__  // don't declare kernel_version in <linux/module.h>
#endif


#include <linux/autoconf.h>

#if defined( CONFIG_MODVERSIONS ) && !defined( MODVERSIONS )
  #define MODVERSIONS
#endif

#if defined( MODVERSIONS )
  #if ( LINUX_VERSION_CODE < KERNEL_VERSION( 2, 6, 0 ) )
    #include <linux/modversions.h>
  #endif
#endif

#include <linux/module.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/poll.h>
#include <linux/interrupt.h>
#include <asm/uaccess.h>
#include <asm/io.h>

#include <linux/proc_fs.h>
#include <linux/init.h>
#include <linux/param.h>


// definitions for clock() support in kernel drivers
extern unsigned long volatile jiffies;
#define clock() jiffies


#define _MBG_INIT_CODE_ATTR           __init


#define _DEFAULT_MBG_TGT_LINUX_USE_PCI_PNP \
  ( LINUX_VERSION_CODE >= KERNEL_VERSION( 2, 4, 0 ) )

#define _DEFAULT_MBG_TGT_LINUX_USE_USB \
  ( ( defined( CONFIG_USB ) || defined( CONFIG_USB_MODULE ) ) && \
  ( LINUX_VERSION_CODE >= KERNEL_VERSION( 2, 6, 0 ) ) )


#define _PCPS_USE_LINUX_CHRDEV \
  ( LINUX_VERSION_CODE >= KERNEL_VERSION( 2, 6, 0 ) )

#define _PCPS_USE_LINUX_MISC_DEV \
  ( !_PCPS_USE_LINUX_CHRDEV )

#define _USE_LINUX_SYSFS \
  ( defined( CONFIG_SYSFS ) )

#define _USE_LINUX_DEVFS \
  ( defined( CONFIG_DEVFS_FS ) )

#define _USE_LINUX_DEVFS_26 \
  ( _USE_LINUX_DEVFS && \
  ( LINUX_VERSION_CODE >= KERNEL_VERSION( 2, 6, 0 ) ) )

#define _USE_LINUX_DEVFS_24 \
  ( _USE_LINUX_DEVFS && \
  ( LINUX_VERSION_CODE >= KERNEL_VERSION( 2, 4, 0 ) ) )


// Starting at some version 2.6.11 or 2.6.12 time out values for the
// USB functions need to be specified in ms, which is the default.
#define _PCPS_USB_TIMEOUT_IS_MS \
  ( LINUX_VERSION_CODE >= KERNEL_VERSION( 2, 6, 12 ) )

// Older Linux kernels expect USB timeout intervals in jiffies, so
// we specify a conversion macro for this case.
#if !_PCPS_USB_TIMEOUT_IS_MS
  #define _pcps_ms_to_usb_timeout( _ms )  ( (_ms) * HZ / 1000 )
#endif

// Early Linux 2.6.x kernels generate a syslog msg on each USB timeout, so the syslog
// would be flooded with USB timeout msgs if we used short timeouts with retries.
#if !defined( _PCPS_USB_FULL_CYCLIC_INTV )
  #define _PCPS_USB_FULL_CYCLIC_INTV \
    ( LINUX_VERSION_CODE < KERNEL_VERSION( 2, 6, 11 ) )
#endif


#if _PCPS_USE_LINUX_CHRDEV
  #include <linux/cdev.h>
#elif _PCPS_USE_LINUX_MISC_DEV
  #include <linux/miscdevice.h>
#endif


#if !defined( _PCPS_USE_LINUX_KTHREAD )
  #define _PCPS_USE_LINUX_KTHREAD \
  ( LINUX_VERSION_CODE >= KERNEL_VERSION( 2, 6, 10 ) )  //##++++
#endif

#if _PCPS_USE_LINUX_KTHREAD
  #include <linux/kthread.h>
#endif


#if !defined( NEW_FASYNC )
  // A third parameter to kill_fasync has been added in kernel 2.3.21,
  // and is _not_ present in _stock_ 2.2.14 and 2.2.15.  However, some
  // distributions patch 2.2.x kernels to add this parameter.
  #define NEW_FASYNC      ( LINUX_VERSION_CODE >= KERNEL_VERSION( 2, 2, 18 ) )
#endif

#if !defined( NEW_FASYNC2 )
  // Type of first parameter of kill_fasync() changed in kernel 2.4.0-test2
  #define NEW_FASYNC2     ( LINUX_VERSION_CODE >= KERNEL_VERSION( 2, 4, 0 ) )
#endif

#if !defined( NEW_WAIT_QUEUE )
  // New wait queue types and macros have been defined around kernel 2.2.18.
  #define NEW_WAIT_QUEUE  ( LINUX_VERSION_CODE >= KERNEL_VERSION( 2, 2, 18 ) )
#endif

#if !defined( NEW_FILE_OPS )
  // A field "owner" has been added to struct file_operations in kernel 2.4.0
  // which needs to be initialized properly.
  #define NEW_FILE_OPS    ( LINUX_VERSION_CODE >= KERNEL_VERSION( 2, 4, 0 ) )
#endif

#if !defined( USE_WAIT_EVENT )
  // Function wait_event_interruptible() has been added in kernel 2.4.20 
  // which replaces the older interruptible_sleep_on() function.
  #define USE_WAIT_EVENT  ( LINUX_VERSION_CODE >= KERNEL_VERSION( 2, 4, 20 ) )
#endif

#if !defined( ESPIPE_BY_VFS )
  // ESPIPE logic has been removed from drivers and is handled by the VFS layer now.
  #define ESPIPE_BY_VFS   ( LINUX_VERSION_CODE >= KERNEL_VERSION( 2, 6, 8 ) )
#endif

#if !defined( FLUSH_WITH_LOCK_OWNER_ID )
  // Starting in kernel 2.6.18 the prototype for the flush function in
  // struct file_operations in fs.h has been modified to take the POSIX
  // lock owner ID as additional parameter.
  // The same patch also defined FL_CLOSE in fs.h, so we check additionally
  // whether this symbol is defined.
  // At least for SUSE SLES10 this has been backported to a kernel which
  // reports to be 2.6.16.
  #define FLUSH_WITH_LOCK_OWNER_ID \
    ( ( LINUX_VERSION_CODE >= KERNEL_VERSION( 2, 6, 18 ) ) \
    || defined( FL_CLOSE ) )
#endif

#if !defined( REQUEST_IRQ_WO_REGS )
  // The prototype of the interrupt handler function has changed around 2.6.19.
  // The newer version does not take a pointer to a register set.
  // This also seems to include linux/bottom_half.h from linux/interrupt.h, 
  // so a criteria for this is also if _LINUX_BH_H is defined after 
  // linux/interrupt.h has been included.
  #define REQUEST_IRQ_WO_REGS \
    ( ( LINUX_VERSION_CODE >= KERNEL_VERSION( 2, 6, 19 ) \
    || defined( _LINUX_BH_H ) ) )
#endif


#if !defined( IRQ_RETVAL )
  // defines for compatibility with older kernels
  typedef void irqreturn_t;
  #define IRQ_NONE
  #define IRQ_HANDLED
//##++  #define IRQ_RETVAL(x)
#endif


// Early hotplug systems have defined __devexit,
// but not __devexit_p.
#if !defined( __devexit_p )
  #define __devexit_p( _x ) _x
#endif

#endif  /* _MBG_LX_H */

